

function gadget:GetInfo()
	return {
		name      = "Map Lua Metal and Geo Spot Placer",
		desc      = "Places metal and geo spots according to lua config file",
		author    = "raaar",
		version   = "v2",
		date      = "2022",
		license   = "PD",
		layer     = -10,
		enabled   = true
	}
end

------------------------------------------------------------
-- Config
------------------------------------------------------------
local MAPSIDE_RESOURCESPOTS = "mapconfig/map_resource_spot_layout.lua"
local MAPSIDE_MAPINFO = "mapinfo.lua"

local spGetTimer = Spring.GetTimer
local spDiffTimers = Spring.DiffTimers

local METAL_MAP_SQUARE_SIZE = 16
local MEX_RADIUS = Game.extractorRadius
local MAP_SIZE_X = Game.mapSizeX
local MAP_SIZE_X_SCALED = MAP_SIZE_X / METAL_MAP_SQUARE_SIZE
local MAP_SIZE_Z = Game.mapSizeZ
local MAP_SIZE_Z_SCALED = MAP_SIZE_Z / METAL_MAP_SQUARE_SIZE

local mapConfig = VFS.FileExists(MAPSIDE_RESOURCESPOTS) and VFS.Include(MAPSIDE_RESOURCESPOTS) or false
local mapInfo = VFS.FileExists(MAPSIDE_MAPINFO) and VFS.Include(MAPSIDE_MAPINFO) or false

local spGetGroundHeight = Spring.GetGroundHeight

-- get metal spot texture name suffix
local function getMetalIdx(metal)
	if (metal > 2.7) then
		return 3
	elseif (metal > 1.8) then
		return 2
	end

	return 1
end

if gadgetHandler:IsSyncedCode() then --------------------- SYNCED - set spot metal values

-- assigns metal to the defined metal spots when the gadget loads
function gadget:Initialize()
	if mapConfig and mapInfo then
		--for key,value in pairs(mapInfo) do
		--	Spring.Echo("HERE "..key.."="..tostring(value))
		--end
	
		Spring.Log(gadget:GetInfo().name, LOG.INFO, "Loading map-side lua metal spot configuration...")
		spots = mapConfig.spots
		metalFactor = 0.43 * 9 / 21

		local gaiaTeamId = Spring.GetGaiaTeamID()
		local features = nil
		local metalIdx = 1
		local xIndex, zIndex, xi, zi
		if (spots and #spots > 0) then
			for i = 1, #spots do
				local spot = spots[i]

				px = spot.x
				pz = spot.z
				metal = spot.metal

				-- place metal for spot
				if (px and pz and metal) then
					
					--Spring.Echo("metal set for x="..px.." z="..pz.." metal="..metal)
					xIndex = math.floor(px / METAL_MAP_SQUARE_SIZE)
					zIndex = math.floor(pz / METAL_MAP_SQUARE_SIZE)

					-- set the metal values
					if xIndex >=0 and zIndex >=0 then
						for dxi = -2, 2 do
							for dzi = -2, 2 do
								if (math.abs(dxi) == 2 and math.abs(dzi) == 2) then 
									-- skip corners
								else
									xi = xIndex + dxi
									zi = zIndex + dzi
		
									if (xi > 0 and xi < MAP_SIZE_X_SCALED and zi > 0 and zi < MAP_SIZE_Z_SCALED ) then							
										Spring.SetMetalAmount(xi,zi,metal * metalFactor *255)
									end
								end
							end
						end
					end
					
					-- generate corrected position for feature
					pxFixed = (xIndex * METAL_MAP_SQUARE_SIZE) + METAL_MAP_SQUARE_SIZE / 2
					pzFixed = (zIndex * METAL_MAP_SQUARE_SIZE) + METAL_MAP_SQUARE_SIZE / 2
					--Spring.Echo("metal spot at x="..pxFixed.." z="..pzFixed.." (x="..px..",z="..pz..")")
					-- check if there is any feature there
					-- if not, place one
					features = Spring.GetFeaturesInCylinder(pxFixed,pzFixed,15)
					if (not features or #features == 0) then
						metalIdx = getMetalIdx(metal)

					
						-- add feature to mark the spot
						-- NOTE : feature no longer has decal, unsynced code draws texture on the ground instead
						--Spring.Echo("metal feature added at x="..px.." z="..pz)
						local fId = Spring.CreateFeature("map_metal_spot"..metalIdx,pxFixed,spGetGroundHeight(pxFixed,pzFixed),pzFixed,math.random(360),gaiaTeamId)
						Spring.SetFeatureAlwaysVisible(fId, true)
						Spring.SetFeatureNoSelect(fId,true)
						Spring.SetFeatureCollisionVolumeData(fId,0, 0, 0, 0, 0, 0,  0, 0, 0 )
						
					end
				end
			end
		end
	end
end

else ------------------------------- UNSYNCED - set spot textures for both metal and geo spots

local imagesToDraw = {}

local images = {
	mapMetalSpot1 = "_metal1.dds",
	mapMetalSpot2 = "_metal2.dds",
	mapMetalSpot2b = "_metal2b.dds",
	mapMetalSpot3 = "_metal3.dds",
	geoSpot = "_geodecal.dds"
}


local MEX_TEXTURE_RADIUS = 64
local GEO_TEXTURE_RADIUS = 64


local glList = nil
local glListRefreshIdx = -1
local refTimer = spGetTimer()

local glColor 						= gl.Color
local glRect						= gl.Rect
local glTexture 					= gl.Texture
local glTexRect 					= gl.TexRect
local glText	 					= gl.Text
local glCreateList	= gl.CreateList
local glDeleteList	= gl.DeleteList
local glCallList	= gl.CallList

local glMatrixMode = gl.MatrixMode
local glPolygonOffset = gl.PolygonOffset
local glCulling = gl.Culling
local glDepthTest = gl.DepthTest
local glPushMatrix = gl.PushMatrix
local glTranslate = gl.Translate
local glRotate = gl.Rotate
local glDrawGroundQuad = gl.DrawGroundQuad
local glPopMatrix = gl.PopMatrix


local spCreateGroundDecal = Spring.CreateGroundDecal
local spDestroyGroundDecal = Spring.DestroyGroundDecal
local spSetGroundDecalPosAndDims = Spring.SetGroundDecalPosAndDims
local spSetGroundDecalRotation = Spring.SetGroundDecalRotation
local spSetGroundDecalTexture = Spring.SetGroundDecalTexture
local spSetGroundDecalAlpha = Spring.SetGroundDecalAlpha

local spGetConfigInt = Spring.GetConfigInt

local groundDecalsPossible = spSetGroundDecalPosAndDims and spSetGroundDecalRotation and spSetGroundDecalTexture and spSetGroundDecalAlpha
local avoidDisplayLists = Platform.glHaveAMD 


function gadget:Initialize()
	if mapConfig and mapInfo then
	
		Spring.Log(gadget:GetInfo().name, LOG.INFO, "Setting map-side lua metal spot textures...")
		spots = mapConfig.spots
		geoSpots = mapConfig.geoSpots
		--Spring.Echo(" spots="..#spots.." geoSpots="..#geoSpots)
		local gaiaTeamId = Spring.GetGaiaTeamID()
		local features = nil
		local metalIdx = 1
		local xIndex, zIndex, xi, zi
		-- metal spots
		if (spots and #spots > 0) then
			for i = 1, #spots do
				local spot = spots[i]

				px = spot.x
				pz = spot.z
				metal = spot.metal

				if (px and pz and metal) then
					-- generate corrected position for texture
					xIndex = math.floor(px / METAL_MAP_SQUARE_SIZE)
					zIndex = math.floor(pz / METAL_MAP_SQUARE_SIZE)
					pxFixed = (xIndex * METAL_MAP_SQUARE_SIZE) + METAL_MAP_SQUARE_SIZE / 2
					pzFixed = (zIndex * METAL_MAP_SQUARE_SIZE) + METAL_MAP_SQUARE_SIZE / 2

					-- get texture to use
					metalIdx = getMetalIdx(metal)
				
					local rotation = 0
					if spot.rot then
						rotation = spot.rot
					else
						rotation = math.random(0, 360)
					end
				
					imagesToDraw[#imagesToDraw+1]={x=pxFixed,y=spGetGroundHeight(pxFixed,pzFixed),z=pzFixed,texture="unittextures/"..images["mapMetalSpot"..metalIdx],rotation=rotation, radius = MEX_TEXTURE_RADIUS}

					if groundDecalsPossible then
						local decalID = spCreateGroundDecal()
						if decalID ~= nil then
							spSetGroundDecalPosAndDims(decalID,pxFixed,pzFixed,MEX_TEXTURE_RADIUS,MEX_TEXTURE_RADIUS)
							spSetGroundDecalRotation(decalID,rotation)
							spSetGroundDecalTexture(decalID,images["mapMetalSpot"..metalIdx],true)
							spSetGroundDecalAlpha(decalID,1)
						end
					end
				end
			end
		end
		-- geo spots
		if (geoSpots and #geoSpots > 0) then
			for i = 1, #geoSpots do
				local spot = geoSpots[i]

				px = spot.x
				pz = spot.z
				name = spot.name

				if (px and pz and name) then
					-- generate corrected position for texture
					xIndex = math.floor(px / METAL_MAP_SQUARE_SIZE)
					zIndex = math.floor(pz / METAL_MAP_SQUARE_SIZE)
					pxFixed = (xIndex * METAL_MAP_SQUARE_SIZE) + METAL_MAP_SQUARE_SIZE / 2
					pzFixed = (zIndex * METAL_MAP_SQUARE_SIZE) + METAL_MAP_SQUARE_SIZE / 2

					local rotation = 0
					if spot.rot then
						rotation = spot.rot
					else
						rotation = math.random(0, 360)
					end

					imagesToDraw[#imagesToDraw+1]={x=pxFixed,y=spGetGroundHeight(pxFixed,pzFixed),z=pzFixed,texture="unittextures/"..images[name],rotation=rotation, radius = GEO_TEXTURE_RADIUS}

					if groundDecalsPossible then					
						local decalID = spCreateGroundDecal()
						if decalID ~= nil then
							spSetGroundDecalPosAndDims(decalID,pxFixed,pzFixed,GEO_TEXTURE_RADIUS,GEO_TEXTURE_RADIUS)
							spSetGroundDecalRotation(decalID,rotation)
							spSetGroundDecalTexture(decalID,images[name],true)
							spSetGroundDecalAlpha(decalID,1)
						end
					end
				end
			end
		end
	end
end

function drawDecals()
	-- Switch to texture matrix mode
	glMatrixMode(GL.TEXTURE)
   
	glPolygonOffset(-25, -2) -- prevents z-fighting somehow
	glCulling(GL.BACK)
	glDepthTest(true)
		
	for _,img in ipairs(imagesToDraw) do
		glTexture(img.texture)
		glColor(1, 1, 1, 1)
		glPushMatrix()
		glTranslate(0.5, 0.5, 0.5)
		glRotate(img.rotation, 0, 0, 1)   
		glDrawGroundQuad(img.x - img.radius, img.z - img.radius, img.x + img.radius, img.z + img.radius, false, -0.5, -0.5, 0.5, 0.5)
		glPopMatrix()
	end

	glTexture(false)
	glDepthTest(false)
	glCulling(false)
	glPolygonOffset(false)
			   
	-- Restore Modelview matrix
	glMatrixMode(GL.MODELVIEW)
end

-- draw decals using GL calls only if engine decals are disabled
function gadget:DrawWorldPreUnit()
	if (not groundDecalsPossible) or (spGetConfigInt("GroundDecals") == 0) or (spGetConfigInt("AdvMapShading") == 0) then
		if avoidDisplayLists then
			drawDecals()
		else	
			-- refresh gl list only once per second
			local refreshIdx = math.floor(spDiffTimers(spGetTimer(),refTimer)*1)
			if (not glList) or refreshIdx ~= glListRefreshIdx then
				if (glList) then
					glDeleteList(glList)
				end 

				glList = glCreateList(drawDecals)
				glListRefreshIdx = refreshIdx
			end
			glCallList(glList)
		end
	end
end


end